clear all
clc

%==========================================================================
% Parameters Setting additional state in which solvency constraints are more binding 
%==========================================================================
borrow_limit    =  0;                  % "1" means borrow up to 100% of PV(labor income)
gama            =  5;                  % CRRA coefficient
beta            = .95;                % Discount Rate
collateral      =  0.9;

%==========================================================================


calibration_choice  =  9;

if calibration_choice== 9 
%==========================================================================
%No STY Effect, base on cal 7
%==========================================================================
Znum       =   2;
Enum       =   2;
Zshocks         =   [0.9602 1.0402 ]';
Zshocks_total   =   repmat(Zshocks,Enum,1);
eta             =   [0.1947 0.1947 0.8053 0.8053]'*2
Eshocks_total   =   eta*collateral;
q_s=0.2740;
p_s=0.7260;
Ztrans = [q_s 1-q_s ; (1-p_s) p_s];

Etrans = [.9450 1- .9450; 1-.9450 .9450];
EZtrans    =  kron(Etrans,Ztrans);
end


%==========================================================================
%convention on numbering states
%==========================================================================
Inv_EZtrans      =   EZtrans^1000;
Inv_EZtrans      =   Inv_EZtrans(1,:);

Inv_Ztrans          =   Ztrans^1000;
Inv_Ztrans          =   Inv_Ztrans(1,:); 

Inv_Etrans          =   Etrans^1000;
Inv_Etrans          =   Inv_Etrans(1,:); 

%==========================================================================
% find the max length of history such that the prob of each state < uppbound
%==========================================================================
probhigh = 0.15;
maxprob  = 1;
Lagnum   = 1;
while maxprob>probhigh;
      Zlag = [];
      for i=1:Lagnum
      AA   = kron([1:Znum]',ones(Znum^(i-1),1));
      Zlag = [repmat(AA,Znum^(Lagnum-i),1) Zlag];
      end
      maxprob = max(prod(Inv_Ztrans(Zlag),2));
      Lagnum = Lagnum+1;
end
%==========================================================================
% Redefine States
%==========================================================================
problow = 0.005;
Lagnum = size(Zlag,2);
Ztotal = size(Zlag,1);
accprob = cumprod(Inv_Ztrans(Zlag),2);
ZlagTr  = zeros(Ztotal,1);

kk = 1;
i  = 1;
clear ZlagRe ZlagTr ProbRe
while i<Ztotal+1;
index = (accprob(i,:)>problow);
lengthindex = sum(index);
ZlagRe{kk,1} = Zlag(i,index==1);
ZlagRe1st(kk,1) = ZlagRe{kk,1}(1,1); 
indexup = find(sum(abs(Zlag(:,1:lengthindex) - repmat(ZlagRe{kk,1},Ztotal,1)),2)==0);
ZlagTr(indexup,1) = kk ;
ProbRe(kk,1) = sum(accprob(indexup,end));
i = max(indexup)+1;
kk = kk+1;
end

ZtotalRe = size(ZlagRe,1);
H = 1.01*ones(ZtotalRe,Znum);
%==========================================================================
% Standard model parameters
%==========================================================================
B_hat       = beta*Ztrans*Zshocks.^(1-gama); % discount factor in stationary economy
B_hat       = B_hat(ZlagRe1st);
%==========================================================================
% Details for the linear grid approix to D(.)
% Mmax and Mmin: max and min levels of the shares
% dimgrid2: grid points on Saving function
% dimgrid3: grid points on the inverse function of Saving function
%==========================================================================
Mmin = 0.001;
Mmax = max(Eshocks_total./collateral)*3;

dimgrid1 = 1000;
dimgrid2 = 1050;
mgrid2   = linspace(Mmin,max(eta*collateral),dimgrid1+1)';
mgrid2   = [mgrid2(1:dimgrid1); linspace(max(eta*collateral),Mmax,dimgrid2-dimgrid1)'];

dimgrid3 = dimgrid2;
mgrid3   = mgrid2;

%==========================================================================
% Random panel which we use in our
% simulations
%==========================================================================
%load C:\YiLi\RA_Job\2008Spring\Perodic_rebalancing\DRAW
load C:\YiLi_Program\Perodic_rebalancing\DRAW
[N,T]=size(DRAW);

% DRAW = rand(N,T);

%==========================================================================
% Initial panel of consumption shares. Note that it will be important to
% convert the shares into weights for the updating and for the computation
% of H. In the matrix SUMMARY we will store (i) the realized h value and (i)
% current z and (iii) lagged z.
%==========================================================================

%==========================================================================
%   computing the twisted probabilities
%   Z_hat: twisted aggregate probability transition
%   Z_hat_total: twisted probability transition for total state
%==========================================================================
for i = 1:Znum,
    Z_hat(i,:) = (Ztrans(i,:).*Zshocks.^(1-gama)')...
                 /(Ztrans(i,:)*Zshocks.^(1-gama));   
end
Z_hat = Z_hat(ZlagRe1st,:);
Z_hat_total = kron(Etrans,Z_hat);

%==========================================================================
% build transition matrix for states
% P_hat_indc: Next possible history aggregate state index matrix (Ztotal by Ztotal)
% P_hat_final: Total Transition Probability Matrix (2*Ztotal by 2*Ztotal)
% CE_hat     : Conditional probabilty of eta' staying in eta'=L, 
%              given eta and z', Znum*Enum by Enum matrix
%              Note that this is not extendable when Enum=3
%==========================================================================
for i = 1:Ztotal;
   for j = 1:Ztotal;
    P_hat_indc(i,j) = (sum(abs(Zlag(i,1:Lagnum-1) - Zlag(j,2:Lagnum))) == 0);
   end
   ZIND(i,:) = find(P_hat_indc(i,:)~=0);
end
    P_tmp = repmat(P_hat_indc,Enum,Enum);
for i = 1:Ztotal*Enum
    EZIND(i,:) = find(P_tmp(i,:)~=0);
end

ZIND_tmp = ZIND;
for i=1:Ztotal
    index = find(ZIND==i);
    ZIND_tmp(index) = ZlagTr(i);
end

for i=1:ZtotalRe
    index = find(ZlagTr==i);
    ZINDRe(i,:) = mean(ZIND_tmp(index,:),1);
end
clear ZIND_tmp
EZlagTr = [ZlagTr ;ZlagTr+ZtotalRe];

EZIND_tmp = EZIND;
for i=1:Ztotal*Enum
    index = find(EZIND==i);
    EZIND_tmp(index) = EZlagTr(i);
end

for i=1:ZtotalRe*Enum
    index = find(EZlagTr==i);
    EZINDRe(i,:) = mean(EZIND_tmp(index,:),1);
end
clear EZIND_tmp
%==========================================================================
P_hat_final = zeros(ZtotalRe*Enum,ZtotalRe*Enum);
for i=1:ZtotalRe*Enum
P_hat_final(i,EZINDRe(i,:)) = Z_hat_total(i,:);
end

CE_hat      =   EZtrans./repmat(Ztrans,Enum,Enum);
CE_hat      =   CE_hat(:,1:Znum); 

%==========================================================================
% Build panel of households
% Z_STATE:  aggregate state
% T_STATE:  Inital history aggregate state
% E_STATE:  idiosyncratic state for each agents
% ZIND:     Aggregate history state index matrix
%==========================================================================
E_STATE        = repmat(uint8(0),N-1,T);
Z_STATE        = repmat(uint8(0),1,T);
T_STATE        = repmat(uint16(0),1,T);
T_STATERe      = repmat(uint16(0),1,T);

Z_STATE(1,1)   = 1;                                        
T_STATE(1,1)   = 1;
T_STATERe(1,1)   = 1;
E_STATE(:,1)   = (DRAW(2:N,1)<Inv_Etrans(1))+... 
                 (1-(DRAW(2:N,1)<Inv_Etrans(1)))*2;

for i=1:T-1
    Z_STATE(1,i+1)   = sum(DRAW(1,i+1)>=cumsum(Ztrans(Z_STATE(1,i),:))')+1;                
    T_STATE(1,i+1)   = ZIND(T_STATE(1,i),Z_STATE(1,i+1));
    T_STATERe(1,i+1)   = ZINDRe(T_STATERe(1,i),Z_STATE(1,i+1));
    Prob  = CE_hat(Z_STATE(1,i)+(E_STATE(:,i)-1)*Znum,Z_STATE(1,i+1));
    E_STATE(:,i+1) = (DRAW(2:N,i+1)<Prob) + (DRAW(2:N,i+1)>=Prob)*2;
    %balance panel
    unbal = sum(E_STATE(:,i+1)==1)-(N-1)/2;
    chk   = (unbal>0) + (unbal<0)*2;
    index = find(E_STATE(:,i+1)==chk);
    pick  = ceil(((N-1)/2+unbal)*rand(abs(unbal),1));
    E_STATE(index(pick),i+1) = 2*(chk==1)+(chk==2);
end

E_STATE        = uint8(E_STATE);
Z_STATE        = uint8(Z_STATE);
T_STATE        = uint16(T_STATE);
T_STATERe      = uint16(T_STATERe);


clear DRAW
save environ_segmented_cal_9


%==========================================================================
% Implied correlation and std for idiosyncratic shock
%==========================================================================
Eta_trans_R = [ CE_hat(1,1)  1 - CE_hat(1,1);
                CE_hat(Znum+1,1)  1 - CE_hat(Znum+1,1)]; 
Eta_inv_R = Eta_trans_R^1000;
Eta_inv_R = Eta_inv_R(1,:);

Eta_rho_R =  Eta_trans_R(1,1)+Eta_trans_R(2,2)-1

Eta_trans_E = [ CE_hat(2,1)  1 - CE_hat(2,1);
                CE_hat(2+Znum,1)  1 - CE_hat(2+Znum,1)]; 
Eta_inv_E = Eta_trans_E^1000;
Eta_inv_E = Eta_inv_E(1,:);

Eta_rho_E =  Eta_trans_E(1,1)+Eta_trans_E(2,2)-1

Z_rho =  sum(diag(Ztrans))-1
Z_inv = Ztrans^1000;
Z_inv = Z_inv(1,:);

Eta_std_E = (Eta_inv_E*([eta(2) ;eta(4)] - Eta_inv_E*[eta(2) ;eta(4)]).^2)^(1/2)
Eta_std_R = (Eta_inv_R*([eta(1) ;eta(3)] - Eta_inv_R*[eta(1) ;eta(3)]).^2)^(1/2)

LnEta_std_E = (Eta_inv_E*(log([eta(2) ;eta(4)]) - Eta_inv_E*log([eta(2) ;eta(4)])).^2)^(1/2)
LnEta_std_R = (Eta_inv_R*(log([eta(1) ;eta(3)]) - Eta_inv_R*log([eta(1) ;eta(3)])).^2)^(1/2)

Z_std = (Z_inv*(Zshocks - Z_inv*Zshocks).^2)^(1/2)

%Individual income growth rate and std in autarky

for i=1:2000
y = (1-collateral)+ collateral*eta((E_STATE(i,:)-1)*2+Z_STATE);
yg = log(y(2:end)./y(1:end-1))+log(Zshocks(Z_STATE(2:end)))+1;
avg_yg(i,1) = mean(yg);
std_yg(i,1) = std(yg);
end
mean(avg_yg)
mean(std_yg)

latex([Eta_rho_R Eta_rho_E Eta_std_R Eta_std_E Z_rho Z_std mean(avg_yg) mean(std_yg)]')
