clear all
clc

%==========================================================================
% Parameters Setting
%==========================================================================
borrow_limit    =  0;           % "1" means borrow up to 100% of PV(labor income)
gama            =  5;           % CRRA coefficient
beta            = .95;          % Time Discount Fatcor
collateral      =  0.9;         % fraction of non-tradable income share

%==========================================================================
% Benchmark calibration on aggregate shocks and idiosyncratic shocks
%==========================================================================
Znum       =   2;                % number of aggregate state
Enum       =   2;                % number of idiosyncratic state
Zshocks         =   [0.9602 1.0402 ]';
Zshocks_total   =   repmat(Zshocks,Enum,1);
eta             =   [0.1947 0.1947 0.8053 0.8053]'*2;
Eshocks_total   =   eta*collateral; % tradable income share
q_s=0.2740;
p_s=0.7260;
Ztrans = [q_s 1-q_s ; (1-p_s) p_s];
Etrans = [.9450 1- .9450; 1-.9450 .9450];
EZtrans    =  kron(Etrans,Ztrans);

%==========================================================================
%Invariant Distribution
%==========================================================================
Inv_EZtrans      =   EZtrans^1000;
Inv_EZtrans      =   Inv_EZtrans(1,:);
Inv_Ztrans          =   Ztrans^1000;
Inv_Ztrans          =   Inv_Ztrans(1,:); 
Inv_Etrans          =   Etrans^1000;
Inv_Etrans          =   Inv_Etrans(1,:); 
%==========================================================================
% Define the aggregate history state by partition on the unconditional
% probability
%==========================================================================
%==========================================================================
% find the max length of history such that the prob of each state < uppbound
%==========================================================================
probhigh = 0.15;
maxprob  = 1;
Lagnum   = 1;
while maxprob>probhigh;
      Zlag = [];
      for i=1:Lagnum
      AA   = kron([1:Znum]',ones(Znum^(i-1),1));
      Zlag = [repmat(AA,Znum^(Lagnum-i),1) Zlag];
      end
      maxprob = max(prod(Inv_Ztrans(Zlag),2));
      Lagnum = Lagnum+1;
end
%==========================================================================
% Redefine States
%==========================================================================
problow = 0.005;
Lagnum = size(Zlag,2);
Ztotal = size(Zlag,1);
accprob = cumprod(Inv_Ztrans(Zlag),2);
kk = 1;
i  = 1;
clear ZlagRe ZlagTr ProbRe
while i<Ztotal+1;
index = (accprob(i,:)>problow);
lengthindex = sum(index);
ZlagRe{kk,1} = Zlag(i,index==1);
ZlagRe1st(kk,1) = ZlagRe{kk,1}(1,1); 
indexup = find(sum(abs(Zlag(:,1:lengthindex) - repmat(ZlagRe{kk,1},Ztotal,1)),2)==0);
ZlagTr(indexup,1) = kk ;
ProbRe(kk,1) = sum(accprob(indexup,end));
i = max(indexup)+1;
kk = kk+1;
end
ZtotalRe = size(ZlagRe,1);

%==========================================================================
% The implied disocunt factor in stationary economy
%==========================================================================
B_hat       = beta*Ztrans*Zshocks.^(1-gama); % discount factor in stationary economy
B_hat       = B_hat(ZlagRe1st);

%==========================================================================
% Computing the twisted probabilities (for stationary economy)
%   Z_hat: twisted aggregate probability transition
%   Z_hat_total: twisted probability transition for total state
%==========================================================================
for i = 1:Znum,
    Z_hat(i,:) = (Ztrans(i,:).*Zshocks.^(1-gama)')...
                 /(Ztrans(i,:)*Zshocks.^(1-gama));   
end
Z_hat = Z_hat(ZlagRe1st,:);
Z_hat_total = kron(Etrans,Z_hat);

%==========================================================================
% Build transition matrix for states
% P_hat_indc: Next possible history aggregate state index matrix (Ztotal by Ztotal)
% P_hat_final: Total Transition Probability Matrix (2*Ztotal by 2*Ztotal)
% CE_hat     : Conditional probabilty of eta' staying in eta'=L, 
%              given eta and z', Znum*Enum by Enum matrix
%              Note that this is not extendable when Enum=3
%==========================================================================
for i = 1:Ztotal;
   for j = 1:Ztotal;
    P_hat_indc(i,j) = (sum(abs(Zlag(i,1:Lagnum-1) - Zlag(j,2:Lagnum))) == 0);
   end
   ZIND(i,:) = find(P_hat_indc(i,:)~=0);
end
    P_tmp = repmat(P_hat_indc,Enum,Enum);

%==========================================================================
% Build transition index matrix for law of motion in aggregate history states
% ZINDRe:      Aggregate history state index transition matrix
% EZINDRe:     Aggregate history and isiosyncratic state index transition matrix
%==========================================================================
for i = 1:Ztotal*Enum
    EZIND(i,:) = find(P_tmp(i,:)~=0);
end

ZIND_tmp = ZIND;
for i=1:Ztotal
    index = find(ZIND==i);
    ZIND_tmp(index) = ZlagTr(i);
end

for i=1:ZtotalRe
    index = find(ZlagTr==i);
    ZINDRe(i,:) = mean(ZIND_tmp(index,:),1);
end
clear ZIND_tmp
EZlagTr = [ZlagTr ;ZlagTr+ZtotalRe];

EZIND_tmp = EZIND;
for i=1:Ztotal*Enum
    index = find(EZIND==i);
    EZIND_tmp(index) = EZlagTr(i);
end

for i=1:ZtotalRe*Enum
    index = find(EZlagTr==i);
    EZINDRe(i,:) = mean(EZIND_tmp(index,:),1);
end
clear EZIND_tmp
%==========================================================================
% CE_hat: the conditional transition probability of idiosyncratic shock
%==========================================================================
P_hat_final = zeros(ZtotalRe*Enum,ZtotalRe*Enum);
for i=1:ZtotalRe*Enum
P_hat_final(i,EZINDRe(i,:)) = Z_hat_total(i,:);
end

CE_hat      =   EZtrans./repmat(Ztrans,Enum,Enum);
CE_hat      =   CE_hat(:,1:Znum); 
%==========================================================================
% Initial guess for the growth rate of aggregate multiplier
%==========================================================================
H = 1.01*ones(ZtotalRe,Znum);
%==========================================================================
% Mmax and Mmin: max and min levels of the shares
% dimgrid2: grid points on Saving function and Consumption function (policy function)
% dimgrid3: grid points on the inverse function of Saving function
%==========================================================================
Mmin = 0.001;
Mmax = max(Eshocks_total./collateral)*3;
dimgrid1 = 1000;
dimgrid2 = 1050;
mgrid2   = linspace(Mmin,max(eta*collateral),dimgrid1+1)';
mgrid2   = [mgrid2(1:dimgrid1); linspace(max(eta*collateral),Mmax,dimgrid2-dimgrid1)'];
dimgrid3 = dimgrid2;
mgrid3   = mgrid2;

%==========================================================================
% Draw Random panel which we use in our simulations
%==========================================================================
N = 3001;
T = 10000;
DRAW = rand(N,T);
%==========================================================================
% Build panel of households
% Z_STATE:   aggregate state
% T_STATE:   Inital history aggregate state
% T_STATERe: Refine history aggregate state
% E_STATE:   idiosyncratic state for each agents
%==========================================================================
E_STATE        = repmat(uint8(0),N-1,T);
Z_STATE        = repmat(uint8(0),1,T);
T_STATE        = repmat(uint16(0),1,T);
T_STATERe      = repmat(uint16(0),1,T);

Z_STATE(1,1)   = 1;                                        
T_STATE(1,1)   = 1;
T_STATERe(1,1)   = 1;
E_STATE(:,1)   = (DRAW(2:N,1)<Inv_Etrans(1))+... 
                 (1-(DRAW(2:N,1)<Inv_Etrans(1)))*2;

for i=1:T-1
    Z_STATE(1,i+1)   = sum(DRAW(1,i+1)>=cumsum(Ztrans(Z_STATE(1,i),:))')+1;                
    T_STATE(1,i+1)   = ZIND(T_STATE(1,i),Z_STATE(1,i+1));
    T_STATERe(1,i+1)   = ZINDRe(T_STATERe(1,i),Z_STATE(1,i+1));
    Prob  = CE_hat(Z_STATE(1,i)+(E_STATE(:,i)-1)*Znum,Z_STATE(1,i+1));
    E_STATE(:,i+1) = (DRAW(2:N,i+1)<Prob) + (DRAW(2:N,i+1)>=Prob)*2;
    %balance panel
    unbal = sum(E_STATE(:,i+1)==1)-(N-1)/2;
    chk   = (unbal>0) + (unbal<0)*2;
    index = find(E_STATE(:,i+1)==chk);
    pick  = ceil(((N-1)/2+unbal)*rand(abs(unbal),1));
    E_STATE(index(pick),i+1) = 2*(chk==1)+(chk==2);
end

E_STATE        = uint8(E_STATE);
Z_STATE        = uint8(Z_STATE);
T_STATE        = uint16(T_STATE);
T_STATERe      = uint16(T_STATERe);

clear DRAW
save environ_cal_9