%==========================================================================
% Given H and updated H by simulating for periodic rebalancing
%==========================================================================
clear all;
clc
load environ_cal_9
load H_all

%==========================================================================
% Define Eshocks for each group
%==========================================================================
Z_std = (Inv_Ztrans*(Zshocks - Inv_Ztrans*Zshocks).^2)^(1/2);
Eshocks_total   =   repmat(Eshocks_total,1,5);
index = [ZlagRe1st ;ZlagRe1st+Znum];
Income_total = Eshocks_total(index,:);

%==========================================================================
% Define the fraction of traders' pool
%==========================================================================
MM = [0     0;
      0.05  0.05;
      0.45  0;
      0.5   0.5;
      0     0.45];
%==========================================================================
% For each case, compute the growth rate of H for each possibel aggregate
% history
%==========================================================================
for case_ind = 1:size(MM,2)
    omega1 = MM(1,case_ind); % fraction of Complete Mertonian trader
    omega2 = MM(2,case_ind); % fraction of Z-complete Mertonian trader
    omega3 = MM(3,case_ind); % fraction of CRB trader
    omega4 = MM(4,case_ind); % fraction of non-participants trader
    omega5 = MM(5,case_ind); % fraction of IRB trader
    l = 3;                   % leverage ratio
    L = l/(1+l);
    port_share = 0.45;       % equity target share 
    K = 3;                   % IRB traders rebalances every K periods     
    Choose = int8([1 2 3; 3 1 2; 2 3 1]+1);
    H = H_all{1,case_ind}
    
gap_H = 1;
count2 =1; 
while gap_H > 0.0005;
    count2
    H_total     = zeros(ZtotalRe,ZtotalRe);
    for i=1:ZtotalRe
        H_total(i,ZINDRe(i,:)) = H(i,:).^gama;
    end
    R_hat_final = repmat(B_hat,Enum,ZtotalRe*Enum).*P_hat_final.*repmat(H_total,Enum,Enum);
             
    % Risk free interest rate
    R_z_final = beta*repmat(Zshocks'.^-gama,ZtotalRe,1).*Ztrans(ZlagRe1st,:).*H.^gama;
    risk_free = sum(R_z_final,2).^(-1);
    
    for i=1:ZtotalRe*Enum
        R_hat_final_tmp{1}(i,:) = R_hat_final(i,EZINDRe(i,:));
    end
    
    % P/d ratio for aggregate consumption
    ac_temp = sum(R_z_final.*repmat(Zshocks',ZtotalRe,1),2);
    zshocks_tmp = zeros(ZtotalRe,ZtotalRe);
       for i = 1:ZtotalRe
         zshocks_tmp(i,ZINDRe(i,:)) = R_z_final(i,:).*Zshocks';
       end  
    ac_pd = inv(eye(ZtotalRe)-zshocks_tmp)*ac_temp;
    
    % Normalized K period Return for leverage claim, risk free rate and diversified claims       
    % K Return for leverage claim

    Return_LCS  = (((ac_pd(ZINDRe) + 1).*repmat(Zshocks',ZtotalRe,1)./repmat(ac_pd,1,Znum)...
            -L*repmat(risk_free,1,Znum))/(1-L))./repmat(Zshocks',ZtotalRe,1);
    Return_RFS  = repmat(risk_free,1,Znum)./repmat(Zshocks',ZtotalRe,1);
    Return_LCS_A{1} = repmat(Return_LCS,Enum,Enum);
    Return_RFS_A{1} = repmat(Return_RFS,Enum,Enum);
    Return_ACS_A{1}  = port_share*Return_LCS_A{1} + (1-port_share)*Return_RFS_A{1};
    H_A{1}       = repmat(H,Enum,Enum);
    pos{1}        = (1:ZtotalRe*Enum)'; 
    pos{2}        = EZINDRe; 
    
    % K period return and H
    for k=2:K
    for i=1:ZtotalRe*Enum
        pos{k+1}(i,:) = reshape(EZINDRe(pos{k}(i,:),:)',1,(Enum*Znum)^k);
        Return_LCS_A{k}(i,:) = kron(Return_LCS_A{k-1}(i,:),ones(1,Enum*Znum)).*reshape(Return_LCS_A{1}(pos{k}(i,:),:)',1,(Enum*Znum)^k);  
        Return_RFS_A{k}(i,:) = kron(Return_RFS_A{k-1}(i,:),ones(1,Enum*Znum)).*reshape(Return_RFS_A{1}(pos{k}(i,:),:)',1,(Enum*Znum)^k);
        H_A{k}(i,:) = kron(H_A{k-1}(i,:),ones(1,Enum*Znum)).*reshape(H_A{1}(pos{k}(i,:),:)',1,(Enum*Znum)^k);
        R_hat_final_tmp{k}(i,:) = kron(R_hat_final_tmp{k-1}(i,:),ones(1,Enum*Znum)).*reshape(R_hat_final_tmp{1}(pos{k}(i,:),:)',1,(Enum*Znum)^k);
    end
        Return_ACS_A{k}  = port_share*Return_LCS_A{k} + (1-port_share)*Return_RFS_A{k};  
    end
        
    NPV_inc = inv(eye(ZtotalRe*Enum)-R_hat_final)*Income_total;
    NPV_agg = inv(eye(ZtotalRe*Enum)-R_hat_final)*collateral*ones(ZtotalRe*Enum,1);
    Dbound = borrow_limit.*NPV_inc;
    
    if max(sum(R_hat_final_tmp{1},2))>1
        disp('discount factor >1')  
        max(sum(R_hat_final_tmp{1},2))
    end
%==========================================================================
% Compute the inital guess for Saving function and actual value of NPV_inc
% 1. Compute saving data: S = (I-beta_hat)^(-1)*(Income-Consumption)
%    Compute NPV_inc = (I-beta_hat)^(-1)*Income
% 2. the borrowing limit is set as a fraction of the PV of future labor 
%    income and called it "Dbound", note that "Dbound" only depends on H
% 3. Mbound: the comsumption cutoffs
% 4. Sav: saving functions
%==========================================================================

if count2==1
Sav_com  = (inv(eye(ZtotalRe*Enum)-R_hat_final)*...
          (repmat(Income_total(:,1),1,dimgrid2)-repmat(mgrid2',ZtotalRe*Enum,1)))';
% Sav_com: saving function for Complete Mertonian traders      
Sav_z    = (inv(eye(ZtotalRe*Enum)-R_hat_final)*...
          (repmat(Income_total(:,2),1,dimgrid2)-repmat(mgrid2',ZtotalRe*Enum,1)))';
% Sav_z: saving function for Z-complete Mertonian traders      
Sav_cap  = (inv(eye(ZtotalRe*Enum)-R_hat_final)*...
          (repmat(Income_total(:,3),1,dimgrid2)-repmat(mgrid2',ZtotalRe*Enum,1)))';
% Sav_cap: saving function for CRB traders            
Sav_bond = (inv(eye(ZtotalRe*Enum)-R_hat_final)*...
          (repmat(Income_total(:,4),1,dimgrid2)-repmat(mgrid2',ZtotalRe*Enum,1)))';
% Sav_bond: saving function for Non-participants      
options = optimset('MaxFunEvals',1e3,'TolFun',1e-10,'LargeScale','off','Display','off');
chkup       = round(0.95*dimgrid2);

Sav_pr_b = [];
% Sav_pr_b: saving function for IRB trader during the rebalancing period      
Sav_pr_nb = [];
% Sav_pr_b: saving function for IRB trader during the non-rebalancing period

end
Cpr_com = []; Cpr_z = []; Cpr_cap = []; Cpr_bond = []; Cpr_b = []; Cpr_nb = [];
% Consumption functions for different types of traders.

%==========================================================================
% Solving the Saving function and policy function for consumption for each
% type of traders
%==========================================================================
%==========================================================================
% Compute Saving Function and Policy function for Complete Mertonian traders
%==========================================================================
if omega1~=0
tic
gap_val_com = 10;
count_com   = 1;
while gap_val_com> 0.001;
    Spr       = zeros(dimgrid2,Znum*Enum);
for i=1:ZtotalRe*Enum
      for j=1:Znum*Enum
      H_next = H(rem(i-1,ZtotalRe)+1,rem(j-1,Znum)+1); 
      Spr(:,j)    = min(interp1(mgrid2,Sav_com(:,EZINDRe(i,j)),mgrid2/H_next,'linear','extrap'),Dbound(EZINDRe(i,j),1));
      end
      Sav_com_new(:,i) = Income_total(i,1) - mgrid2 + Spr*R_hat_final_tmp{1}(i,:)';
end

   gap_val_com = max(max(abs(Sav_com(1:chkup,:)-Sav_com_new(1:chkup,:))));
   Sav_com     = Sav_com_new;
   count_com   = count_com+1;
end
disp('time for updating Saving funtion complete contingent claims')
toc
tic
for i=1:ZtotalRe*Enum
      for j=1:Znum*Enum
      H_next = H(rem(i-1,ZtotalRe)+1,rem(j-1,Znum)+1); 
      index  = (interp1(mgrid2,Sav_com(:,EZINDRe(i,j)),mgrid2/H_next,'linear','extrap')<Dbound(EZINDRe(i,j),1));
      Cpr_com{i}(:,j) = index.*(mgrid2/H_next) + (1-index)*mgrid2(sum(1-index))/H_next;
      end
end
toc
end
%==========================================================================
% End for computing saving function and policy function for Complete
% Mertonian traders
%==========================================================================
%==========================================================================
% Compute Saving Function and Policy function for Z-Complete Mertonian traders
%==========================================================================
if omega2~=0
tic
gap_val_z = 10;
count_z  = 1;
while gap_val_z> 0.001;
Dbound_low_z = min(Sav_z);
Spr       = zeros(ZtotalRe*Enum,Znum,dimgrid2);
for i=1:ZtotalRe*Enum
    for j=1:Znum
        clear Savpr_tmp c_tmp
        Prob   = CE_hat(ceil(i/(ZtotalRe/Znum)),j);
        H_next = H(rem(i-1,ZtotalRe)+1,j);
        Dbound_low_tmp = Dbound_low_z(EZINDRe(i,[j j+Znum]));
        Savpr_tmp = linspace(Dbound(EZINDRe(i,j),2),min(Dbound_low_tmp),dimgrid3)';
        c_tmp     = H_next*(Prob*max(interp1(Sav_z(:,EZINDRe(i,j)),mgrid2,Savpr_tmp,'linear','extrap'),0).^(-gama)...
              +(1-Prob)*interp1(Sav_z(:,EZINDRe(i,j+Znum)),mgrid2,Savpr_tmp).^(-gama)).^(-1/gama);
        pick = find(c_tmp>Mmin);
        Savpr_tmp = Savpr_tmp(pick);
        c_tmp     = c_tmp(pick);
        Spr(i,j,:) = (mgrid2>c_tmp(1)).*interp1(c_tmp,Savpr_tmp,mgrid2,'linear','extrap')...
                     +(mgrid2<=c_tmp(1)).*Dbound(EZINDRe(i,j),2);
    end
end

for j = 1:dimgrid2 
    Sav_z_new(j,:) = Income_total(:,2) - mgrid2(j) + sum(repmat(Spr(:,:,j),1,Enum).*R_hat_final_tmp{1},2);
end
   gap_val_z = max(max(abs(Sav_z(1:chkup,:)-Sav_z_new(1:chkup,:))));
   Sav_z      = Sav_z_new;
   count_z   = count_z+1;
end
disp('time for updating Saving funtion z')
toc
end
%==========================================================================
% End for updating saving function z contingetnt
%==========================================================================
%==========================================================================
% Compute saving function CRB traders
%==========================================================================
if omega3~=0||count2==1
tic
gap_val_cap = 10;
count_cap  = 1;
while gap_val_cap> 0.0005; 
clear cprbound_cap
Spr       = zeros(ZtotalRe*Enum,dimgrid2);
Dbound_low_cap = min(Sav_cap);
for i=1:ZtotalRe*Enum
    clear Spr_tmp c_tmp cpr_tmp Dbound_tmp
    Dbound_tmp = Return_ACS_A{1}(i,:)'.*(min(Dbound(EZINDRe(i,:),3)./Return_ACS_A{1}(i,:)'));
    Wbound     =  min(Dbound(EZINDRe(i,:),3)./Return_ACS_A{1}(i,:)');    
    W_tmp  = linspace(Wbound,min(Dbound_low_cap),dimgrid3)';   
    Spr_tmp = repmat(W_tmp,1,Enum*Znum).*repmat(Return_ACS_A{1}(i,:),dimgrid3,1);
      for j=1:Znum*Enum
          cpr_tmp(:,j)   =  max(interp1(Sav_cap(:,EZINDRe(i,j)),mgrid2,Spr_tmp(:,j),'linear','extrap'),0.95*Mmin);
      end
    c_tmp   = ((repmat(H_A{1}(i,:),dimgrid3,1).*cpr_tmp).^(-gama)*(R_hat_final_tmp{1}(i,:).*Return_ACS_A{1}(i,:))').^(-1/gama);
    pick    = find(c_tmp>Mmin);
    W_tmp   = W_tmp(pick);
    Spr_tmp   = Spr_tmp(pick);
    c_tmp   = c_tmp(pick);
    Spr(i,:) = ((mgrid2>c_tmp(1)).*interp1(c_tmp,W_tmp,mgrid2,'linear','extrap')...
                     +(mgrid2<=c_tmp(1)).*W_tmp(1));                 
end
for j = 1:dimgrid2
    Sav_cap_new(j,:) = Income_total(:,3) - mgrid2(j) + Spr(:,j);
end
   gap_val_cap = max(max(abs(Sav_cap(1:chkup,:)-Sav_cap_new(1:chkup,:))));
   Sav_cap      = Sav_cap_new;
   count_cap   = count_cap+1;
end 
disp('time for updating Saving funtion cap only')
toc
end
%==========================================================================
% End for updating saving function capital only
%==========================================================================
%==========================================================================
% Compute saving function of non-participants
%==========================================================================
if omega4~=0||count2==1
tic
gap_val_bond = 10;
count_bond  = 1;
while gap_val_bond> 0.001;
 clear cprbound_bond
 Spr       = zeros(ZtotalRe*Enum,dimgrid2);
 Dbound_low_bond = min(Sav_bond);
 for i=1:ZtotalRe*Enum
    clear W_tmp Spr_tmp c_tmp Dbound_tmp cpr_tmp
    Dbound_tmp = min(Dbound(EZINDRe(i,:),4).*(Zshocks_total./min(Zshocks)));
    W_tmp  = linspace(Dbound_tmp,min(Dbound_low_bond),dimgrid3)';
    Spr_tmp = repmat(W_tmp,1,Enum*Znum).*repmat(Return_RFS_A{1}(i,:),dimgrid3,1);
      for j=1:Znum*Enum
          cpr_tmp(:,j)   =  max(interp1(Sav_bond(:,EZINDRe(i,j)),mgrid2,Spr_tmp(:,j),'linear','extrap'),0.95*Mmin);
      end
    c_tmp   = ((repmat(H_A{1}(i,:),dimgrid3,1).*cpr_tmp).^(-gama)*(R_hat_final_tmp{1}(i,:).*Return_RFS_A{1}(i,:))').^(-1/gama); 
    pick    = find(c_tmp>Mmin);
    W_tmp   = W_tmp(pick);
    c_tmp   = c_tmp(pick);
    Spr(i,:) = ((mgrid2>c_tmp(1)).*interp1(c_tmp,W_tmp,mgrid2,'linear','extrap')+(mgrid2<=c_tmp(1)).*W_tmp(1));
end

for j = 1:dimgrid2
    Sav_bond_new(j,:) = Income_total(:,4) - mgrid2(j) + Spr(:,j);
end
   gap_val_bond = max(max(abs(Sav_bond(1:chkup,:)-Sav_bond_new(1:chkup,:))));
   Sav_bond     = Sav_bond_new;
   count_bond   = count_bond+1;
end
disp('time for updating Saving funtion bond only')
toc
end
%==========================================================================
% End for updating saving function bond only
%==========================================================================

%==========================================================================
% Compute saving function for periodic rebalancing investor 
%==========================================================================
if omega5~=0
tic
tt=clock;
tt(4:6)
gap_val_pr = 10;
count_pr  = 1;
bindc = zeros(ZtotalRe*Enum,1);

if count2==1
 Sav_pr_b  = Sav_cap;
 Sav_pr_nb = cell(K,ZtotalRe*Enum);
  for k=2:K;
   for i=1:ZtotalRe*Enum
       Sav_pr_nb{k,i} = Sav_bond(:,pos{k}(i,:))*(1-port_share); 
   end
  end 
end
    
for i=1:ZtotalRe*Enum
bindc(i,1) = interp1(Sav_pr_b(:,i),mgrid2,0);
end

for k=2:K;
   for i=1:ZtotalRe*Enum
       for j=1:cols(Sav_pr_nb{k,i})
       bindc_nb{k,i}(j,1) = interp1(Sav_pr_nb{k,i}(:,j),mgrid2,0,'linear','extrap');
       end
   end
end

while gap_val_pr> 0.01&&count_pr<100;
 clear cprbound_bond
%==========================================================================
% For non-rebalancing period saving function
%==========================================================================
Spr       = zeros(ZtotalRe*Enum,dimgrid2);
Dbound_low_pr = min(Sav_pr_b);

for i=1:ZtotalRe*Enum
    clear W_tmp Spr_tmp c_tmp Dbound_tmp cpr_tmp W_tmp_eq W_tmp_rf
    Dbound_tmp = min(Dbound(EZINDRe(i,:),5).*(Zshocks_total./min(Zshocks)));

    W_tmp{1}  = linspace(Dbound_tmp,min(Dbound_low_pr),dimgrid3)';
    W_tmp_eq{1} = port_share*W_tmp{1};
    W_tmp_rf{1} = (1-port_share)*W_tmp{1};
       
    % Non-rebalancing periods
    for k=2:K
        S_eq_tmp{k} = kron(W_tmp_eq{k-1},ones(1,Enum*Znum)).*repmat(reshape(Return_LCS_A{1}(pos{k-1}(i,:),:)',1,(Enum*Znum)^(k-1)),dimgrid2,1);  
        S_rf_tmp{k} = kron(W_tmp_rf{k-1},ones(1,Enum*Znum)).*repmat(reshape(Return_RFS_A{1}(pos{k-1}(i,:),:)',1,(Enum*Znum)^(k-1)),dimgrid2,1); 
        for j=1:size(S_rf_tmp{k},2)
          c_tmp{k}(:,j)   =  max(interp1([Sav_pr_nb{k,i}(:,j);0],[mgrid2; bindc_nb{k,i}(j,1)],S_rf_tmp{k}(:,j),'linear','extrap'),0.95*Mmin);
        end
        W_tmp_rf{k} = S_rf_tmp{k} - repmat(Eshocks_total(:,5)',dimgrid2,(Enum*Znum)^(k-2)) + c_tmp{k};
        W_tmp_eq{k} = S_eq_tmp{k};
        W_tmp{k}    = W_tmp_rf{k}+W_tmp_eq{k};
    end
    % Rebalancing periods
        S_eq_tmp{K+1} = kron(W_tmp_eq{K},ones(1,Enum*Znum)).*repmat(reshape(Return_LCS_A{1}(pos{K}(i,:),:)',1,(Enum*Znum)^(K)),dimgrid2,1);  
        S_rf_tmp{K+1} = kron(W_tmp_rf{K},ones(1,Enum*Znum)).*repmat(reshape(Return_RFS_A{1}(pos{K}(i,:),:)',1,(Enum*Znum)^(K)),dimgrid2,1);
        S_tmp{K+1} = S_eq_tmp{K+1} + S_rf_tmp{K+1};
        for j=1:size(S_tmp{K+1},2)
          c_tmp{K+1}(:,j)   =  max(interp1([Sav_pr_b(:,pos{K+1}(i,j));0],[mgrid2; bindc(pos{K+1}(i,j))],S_tmp{K+1}(:,j),'linear','extrap'),0.95*Mmin);                          
        end
        W_tmp{K+1} = S_tmp{K+1} + repmat(Income_total(pos{K+1}(i,:),5)',dimgrid2,1) - c_tmp{K+1};
        
    %======================================================================
        for k=2:K
        zeta{k}   = repmat(zeros(1,1),dimgrid2,(Enum*Znum)^k);  
        for j=1:(Enum*Znum)^(k-1)
            index = [1:Enum*Znum] + (j-1)*Enum*Znum;
            c_tmp1{k}(:,j)   = ((repmat(H_A{1}(pos{k}(i,j),:),dimgrid3,1).*c_tmp{k+1}(:,index)).^(-gama)...
                *(R_hat_final_tmp{1}(pos{k}(i,j),:).*Return_RFS_A{1}(pos{k}(i,j),:))').^(-1/gama);
            [V b_state] = max(W_tmp{k+1}(1,index));
            b_state1 = index(b_state);
            index2 = find(sum(W_tmp{k+1}(:,index)>-0.1,2)~=0);
            zeta{k}(index2,b_state1) =  max(c_tmp{k}(index2,j).^-gama - c_tmp1{k}(index2,j).^-gama,0)...
                                   /((R_hat_final_tmp{1}(pos{k}(i,j),b_state).*Return_RFS_A{1}(pos{k}(i,j),b_state)));                               
        end
        end
        zeta_all = zeta{K};
        for k=2:K-1
        zeta_all = zeta_all + kron(zeta{k},ones(1,(Enum*Znum)^(K-k)));
        end
        
    %====================================================================== 
      % UPDATING C_TMP START HERE
        c_tmp{1}   = (((repmat(H_A{K}(i,:),dimgrid3,1).*c_tmp{K+1}).^(-gama)+zeta_all)*(R_hat_final_tmp{K}(i,:).*Return_ACS_A{K}(i,:))').^(-1/gama);
        pick       = find(c_tmp{1}>Mmin);
        W_tmptmp   = W_tmp{1}(pick);
        c_tmptmp   = c_tmp{1}(pick);
        Spr(i,:) = ((mgrid2>c_tmptmp(1)).*interp1(c_tmptmp,W_tmptmp,mgrid2,'linear','extrap')+(mgrid2<=c_tmptmp(1)).*W_tmptmp(1));
        
        
        for k=2:K
          k_cur = K+2-k;
        for j=1:(Enum*Znum)^(k_cur-1)
          jpick = [1:Enum*Znum]+(j-1)*Enum*Znum;
          jpos  = pos{k_cur}(i,j);
          c_tmp{k_cur}(:,j)   =  (((repmat(H_A{1}(jpos,:),dimgrid3,1).*c_tmp{k_cur+1}(:,jpick)).^(-gama)+zeta{k_cur}(:,jpick))*...
                                         (R_hat_final_tmp{1}(jpos,:).*Return_RFS_A{1}(jpos,:))').^(-1/gama);
          pick    = find(c_tmp{k_cur}(:,j)>Mmin);
          W_tmptmp_rf = W_tmp_rf{k_cur}(pick,j);
          c_tmptmp    = c_tmp{k_cur}(pick,j);
          W_rf{k_cur}(j,:) = ((mgrid2>c_tmptmp(1)).*interp1(c_tmptmp,W_tmptmp_rf,mgrid2,'linear','extrap')...
                                                                    +(mgrid2<=c_tmptmp(1)).*W_tmptmp_rf(1));
        end
        end

        for k=2:K
        for j = 1:dimgrid2    
        Sav_pr_nb_new{k,i}(j,:) = (repmat(Eshocks_total(:,5),(Enum*Znum)^(k-2),1) - mgrid2(j) + W_rf{k}(:,j))'; 
        end
        end

 for j = 1:dimgrid2
     Sav_pr_b_new(j,:) = Income_total(:,5) - mgrid2(j) + Spr(:,j);
 end

end 
for i=1:ZtotalRe*Enum
    for k=2:K
    max_nb_tmp(k-1) = max(max(abs(Sav_pr_nb{k,i} - Sav_pr_nb_new{k,i})));
    end
    max_nb(i) = max(max_nb_tmp);
end

% gap between Sav_pr_b
boundlow = find(mgrid2>2);
boundlow = boundlow(1);
max_b = max(max(abs(Sav_pr_b(1:boundlow,:) - Sav_pr_b_new(1:boundlow,:))));

[val loc1] = max(abs(Sav_pr_b - Sav_pr_b_new));
[val loc2] = max(val);
loc = loc1(loc2);
count_pr = count_pr+1;
disp(max_b)
disp([loc count_pr])

gap_val_pr = max_b;
Sav_pr_nb = Sav_pr_nb_new;
Sav_pr_b = Sav_pr_b_new;

for i=1:ZtotalRe*Enum
bindc(i,1) = interp1(Sav_pr_b(:,i),mgrid2,0);
end
for k=2:K;
   for i=1:ZtotalRe*Enum
       for j=1:cols(Sav_pr_nb{k,i})
       bindc_nb{k,i}(j,1) = interp1(Sav_pr_nb{k,i}(:,j),mgrid2,0,'linear','extrap');
       end
   end
end

end
toc
end % end if
%==========================================================================
% end of computing saving function for periodic rebalancing investor 
%==========================================================================


%==========================================================================
% Computing consumption function for periodic rebalancing investor 
%==========================================================================
if omega5~=0
%==========================================================================
% c. perodic rebalancing
%==========================================================================
tic
 for i=1:ZtotalRe*Enum
    clear W_tmp Spr_tmp c_tmp Dbound_tmp cpr_tmp
    Dbound_tmp = min(Dbound(EZINDRe(i,:),5).*(Zshocks_total./min(Zshocks)));
    W_tmp{1}  = linspace(Dbound_tmp,min(Dbound_low_pr),dimgrid3)';
    W_tmp_eq{1} = port_share*W_tmp{1};
    W_tmp_rf{1} = (1-port_share)*W_tmp{1};
       
    % Non-rebalancing periods
    for k=2:K
        S_eq_tmp{k} = kron(W_tmp_eq{k-1},ones(1,Enum*Znum)).*repmat(reshape(Return_LCS_A{1}(pos{k-1}(i,:),:)',1,(Enum*Znum)^(k-1)),dimgrid2,1);  
        S_rf_tmp{k} = kron(W_tmp_rf{k-1},ones(1,Enum*Znum)).*repmat(reshape(Return_RFS_A{1}(pos{k-1}(i,:),:)',1,(Enum*Znum)^(k-1)),dimgrid2,1); 
        for j=1:size(S_rf_tmp{k},2)
          c_tmp{k}(:,j)   =  max(interp1([Sav_pr_nb{k,i}(:,j);0],[mgrid2; bindc_nb{k,i}(j,1)],S_rf_tmp{k}(:,j),'linear','extrap'),0.95*Mmin);          
        end
        W_tmp_rf{k} = S_rf_tmp{k} - repmat(Eshocks_total(:,5)',dimgrid2,(Enum*Znum)^(k-2)) + c_tmp{k};
        W_tmp_eq{k} = S_eq_tmp{k};
        W_tmp{k}    = W_tmp_rf{k}+W_tmp_eq{k};
    end
    % Rebalancing periods
        S_eq_tmp{K+1} = kron(W_tmp_eq{K},ones(1,Enum*Znum)).*repmat(reshape(Return_LCS_A{1}(pos{K}(i,:),:)',1,(Enum*Znum)^(K)),dimgrid2,1);  
        S_rf_tmp{K+1} = kron(W_tmp_rf{K},ones(1,Enum*Znum)).*repmat(reshape(Return_RFS_A{1}(pos{K}(i,:),:)',1,(Enum*Znum)^(K)),dimgrid2,1);
        S_tmp{K+1} = S_eq_tmp{K+1} + S_rf_tmp{K+1};
        for j=1:size(S_tmp{K+1},2)
          c_tmp{K+1}(:,j)   =  max(interp1(Sav_pr_b(:,pos{K+1}(i,j)),mgrid2,S_tmp{K+1}(:,j),'linear','extrap'),0.95*Mmin);                          
          c_tmp{K+1}(:,j)   =  max(interp1([Sav_pr_b(:,pos{K+1}(i,j));0],[mgrid2; bindc(pos{K+1}(i,j))],S_tmp{K+1}(:,j),'linear','extrap'),0.95*Mmin);     
        end
        W_tmp{K+1} = S_tmp{K+1} + repmat(Income_total(pos{K+1}(i,:),5)',dimgrid2,1) - c_tmp{K+1};
        
    %======================================================================
        for k=2:K
        zeta{k}   = repmat(zeros(1,1),dimgrid2,(Enum*Znum)^k);  
        for j=1:(Enum*Znum)^(k-1)
            index = [1:Enum*Znum] + (j-1)*Enum*Znum;
            c_tmp1{k}(:,j)   = ((repmat(H_A{1}(pos{k}(i,j),:),dimgrid3,1).*c_tmp{k+1}(:,index)).^(-gama)...
                *(R_hat_final_tmp{1}(pos{k}(i,j),:).*Return_RFS_A{1}(pos{k}(i,j),:))').^(-1/gama);
            [V b_state] = max(W_tmp{k+1}(1,index));
            b_state1 = index(b_state);
            index2 = find(sum(W_tmp{k+1}(:,index)>-0.1,2)~=0);
            zeta{k}(index2,b_state1) =  max(c_tmp{k}(index2,j).^-gama - c_tmp1{k}(index2,j).^-gama,0)...
                                   /((R_hat_final_tmp{1}(pos{k}(i,j),b_state).*Return_RFS_A{1}(pos{k}(i,j),b_state)));                               
        end
        end
        zeta_all = zeta{K};
        for k=2:K-1
        zeta_all = zeta_all + kron(zeta{k},ones(1,(Enum*Znum)^(K-k)));
        end

    %======================================================================   
        c_tmp{1}   = (((repmat(H_A{K}(i,:),dimgrid3,1).*c_tmp{K+1}).^(-gama)+zeta_all)*(R_hat_final_tmp{K}(i,:).*Return_ACS_A{K}(i,:))').^(-1/gama);
        pick       = find(c_tmp{1}>Mmin);
        c_tmptmp   = c_tmp{1}(pick);
        for j=1:(Enum*Znum)^K
            cpr_tmptmp = c_tmp{K+1}(pick,j);
            Cpr_b{i}(:,j) = (mgrid2<=c_tmptmp(1))*interp1(c_tmptmp,cpr_tmptmp,c_tmptmp(1),'linear','extrap')+...
                   (mgrid2>c_tmptmp(1)).*interp1(c_tmptmp,cpr_tmptmp,mgrid2,'linear','extrap');
        end
        
        for k=2:K
          k_cur = K+2-k;
        for j=1:size(S_rf_tmp{k_cur},2)
          jpick = [1:Enum*Znum]+(j-1)*Enum*Znum;
          jpos  = pos{k_cur}(i,j);
          c_tmp{k_cur}(:,j)   =  (((repmat(H_A{1}(jpos,:),dimgrid3,1).*c_tmp{k_cur+1}(:,jpick)).^(-gama)+zeta{k_cur}(:,jpick))*(R_hat_final_tmp{1}(jpos,:).*Return_RFS_A{1}(jpos,:))').^(-1/gama);
          cpr_tmptmp = c_tmp{k_cur}(pick,j);
          Cpr_nb{k_cur,i}(:,j)  = (mgrid2<=c_tmptmp(1))*interp1(c_tmptmp,cpr_tmptmp,c_tmptmp(1),'linear','extrap')+...
                           (mgrid2>c_tmptmp(1)).*interp1(c_tmptmp,cpr_tmptmp,mgrid2,'linear','extrap');
        end
        end     

 end
toc
end
%==========================================================================
% End of computing IRB's consumption functions  
%==========================================================================

%==========================================================================
% Compute consumption policy function
%==========================================================================
%==========================================================================
% Compute Mertonian Z-complete traders
%==========================================================================
if omega2~=0
tic 
clear cprbound_z
for i=1:ZtotalRe*Enum
    for j=1:Znum
        Prob   = CE_hat(ceil(i/(ZtotalRe/Znum)),j);
        H_next = H(rem(i-1,ZtotalRe)+1,j);
        Dbound_low_tmp = Dbound_low_z(EZINDRe(i,[j j+Znum])); 
        Savpr_tmp = linspace(Dbound(EZINDRe(i,j),2),max(Dbound_low_tmp),dimgrid3)';
        cpr1  = max(interp1(Sav_z(:,EZINDRe(i,j)),mgrid2,Savpr_tmp,'linear','extrap'),0);
        cpr2  = max(interp1(Sav_z(:,EZINDRe(i,j+Znum)),mgrid2,Savpr_tmp,'linear','extrap'),0);
        c_tmp = H_next*(Prob*cpr1.^(-gama)+(1-Prob)*cpr2.^(-gama)).^(-1/gama);
        pick    = find(c_tmp>Mmin);
        c_tmp   = c_tmp(pick);
        cpr1    = cpr1(pick);
        cpr2    = cpr2(pick);
        Cpr_z{i}(:,j) = (mgrid2<=c_tmp(1))*interp1(c_tmp,cpr1,c_tmp(1),'linear','extrap')+...
           (mgrid2>c_tmp(1)).*interp1(c_tmp,cpr1,mgrid2,'linear','extrap');
        Cpr_z{i}(:,j+Znum) = (mgrid2<=c_tmp(1))*interp1(c_tmp,cpr2,c_tmp(1),'linear','extrap')+...
           (mgrid2>c_tmp(1)).*interp1(c_tmp,cpr2,mgrid2,'linear','extrap');
    end
end
toc
end
%==========================================================================
% End: Mertonian Z-complete traders 
%==========================================================================
%==========================================================================
% Compute CRB traders 
%==========================================================================
if omega3~=0
 tic
 clear cprbound_cap
for i=1:ZtotalRe*Enum
    clear Spr_tmp c_tmp cpr_tmp
    Dbound_tmp = Return_ACS_A{1}(i,:)'.*(min(Dbound(EZINDRe(i,:),3)./Return_ACS_A{1}(i,:)'));
    Wbound     =  min(Dbound(EZINDRe(i,:),3)./Return_ACS_A{1}(i,:)');
    W_tmp  = linspace(Wbound,min(Dbound_low_cap),dimgrid3)';   
    Spr_tmp = repmat(W_tmp,1,Enum*Znum).*repmat(Return_ACS_A{1}(i,:),dimgrid3,1);    
      for j=1:Znum*Enum
          cpr_tmp(:,j)   =  max(interp1(Sav_cap(:,EZINDRe(i,j)),mgrid2,Spr_tmp(:,j),'linear','extrap'),0.9*Mmin);
      end
    c_tmp   = ((repmat(H_A{1}(i,:),dimgrid3,1).*cpr_tmp).^(-gama)*(R_hat_final_tmp{1}(i,:).*Return_ACS_A{1}(i,:))').^(-1/gama);
    pick    = find(c_tmp>Mmin);
    c_tmp     = c_tmp(pick);
    cpr_tmp   = cpr_tmp(pick,:);
       for j=1:Znum*Enum
           Cpr_cap{i}(:,j) = (mgrid2<=c_tmp(1))*interp1(c_tmp,cpr_tmp(:,j),c_tmp(1),'linear','extrap')+...
                (mgrid2>c_tmp(1)).*interp1(c_tmp,cpr_tmp(:,j),mgrid2,'linear','extrap');
       end 
end
toc
end
%==========================================================================
% End: CRB traders 
%==========================================================================
%==========================================================================
% Compute Non-participants 
%==========================================================================
if omega4~=0
clear cprbound_bond
tic  
for i=1:ZtotalRe*Enum
    clear Dbound_tmp Spr_tmp cpr_tmp c_tmp W_tmp
    Dbound_tmp = min(Dbound(EZINDRe(i,:),4).*(Zshocks_total./min(Zshocks)));
    W_tmp  = linspace(Dbound_tmp,max(Dbound_low_bond),dimgrid3)';
    Spr_tmp = repmat(W_tmp,1,Enum*Znum)./repmat(((Zshocks_total./min(Zshocks))'),dimgrid3,1);
      for j=1:Znum*Enum
          cpr_tmp(:,j)   =  max(interp1(Sav_bond(:,EZINDRe(i,j)),mgrid2,Spr_tmp(:,j),'linear','extrap'),0.9*Mmin);
      end
    c_tmp   = ((repmat(H_A{1}(i,:),dimgrid3,1).*cpr_tmp).^(-gama)*(R_hat_final_tmp{1}(i,:).*Return_RFS_A{1}(i,:))').^(-1/gama); 
    pick    = find(c_tmp>Mmin);
    c_tmp     = c_tmp(pick);
    cpr_tmp   = cpr_tmp(pick,:);
       for j=1:Znum*Enum
           Cpr_bond{i}(:,j) = (mgrid2<=c_tmp(1))*interp1(c_tmp,cpr_tmp(:,j),c_tmp(1),'linear','extrap')+...
                (mgrid2>c_tmp(1)).*interp1(c_tmp,cpr_tmp(:,j),mgrid2,'linear','extrap');
       end 
end
toc
end
%==========================================================================
% End: Non-participants 
%==========================================================================
%==========================================================================
% Simulation
%==========================================================================
%==========================================================================
% Simulate the economy by T periods and N agnets for each type of traders
%==========================================================================
tic
  clear C_STATE cpr_tmpN5
  H_STATE =repmat(single(0),size(T_STATERe));
  h_update = repmat(single(1),T,1);
  C_STATE_N5 = repmat(single(0),N-1,K);
  C_STATE_TMP = repmat(single(0),N-1,5);
  
  cpr_tmpsim = cell(K,K+1);
    for kk=1:K
      for k=1:K+1
      cpr_tmpsim{kk,k} = ones(N-1,1);
      end
    end
  
  for t=1:T-K
      pick = [(E_STATE(:,t)==1).*(E_STATE(:,t+1)==1) (E_STATE(:,t)==1).*(E_STATE(:,t+1)==2)...
             (E_STATE(:,t)==2).*(E_STATE(:,t+1)==1) (E_STATE(:,t)==2).*(E_STATE(:,t+1)==2)];
         
      Agg_pick =  [Z_STATE(1,t+1) Z_STATE(1,t+1)+Znum];
      
      if omega1~=0      
         cp_tmp       = [Cpr_com{T_STATERe(1,t)}(:,Agg_pick) Cpr_com{T_STATERe(1,t)+ZtotalRe}(:,Agg_pick)];
         C_STATE_TMP(:,1)  = sum(interp1(mgrid2,cp_tmp,C_STATE_TMP(:,1),'linear','extrap').*pick,2);
      end
      
      if omega2~=0
         cp_tmp     = [Cpr_z{T_STATERe(1,t)}(:,Agg_pick) Cpr_z{T_STATERe(1,t)+ZtotalRe}(:,Agg_pick)];
         C_STATE_TMP(:,2)  = sum(interp1(mgrid2,cp_tmp,C_STATE_TMP(:,2),'linear','extrap').*pick,2);
      end
      
      if omega3~=0
         cp_tmp     = [Cpr_cap{T_STATERe(1,t)}(:,Agg_pick) Cpr_cap{T_STATERe(1,t)+ZtotalRe}(:,Agg_pick)];
         C_STATE_TMP(:,3)  = sum(interp1(mgrid2,cp_tmp,C_STATE_TMP(:,3),'linear','extrap').*pick,2);
      end
      
      if omega4~=0
         cp_tmp     = [Cpr_bond{T_STATERe(1,t)}(:,Agg_pick) Cpr_bond{T_STATERe(1,t)+ZtotalRe}(:,Agg_pick)];
         C_STATE_TMP(:,4)  = sum(interp1(mgrid2,cp_tmp,C_STATE_TMP(:,4),'linear','extrap').*pick,2);
      end
      
      if omega5~=0
          kk = rem(t-1,K)+1;
          if t>K
          C_STATE_N5(:,kk) = C_STATE_N5(:,kk)/h_update(t-1);
          end
             Agg_pick2{1} = Agg_pick;
             pick2{1}     = pick;
             
            for k=2:K
                Agg_pick2{k} = kron(double(Agg_pick2{k-1}-1)*(Enum*Znum),ones(1,Enum)) + double(repmat([Z_STATE(1,t+k) Z_STATE(1,t+k)+Znum],1,Enum^(k-1)));
                for ii=1:size(pick2{k-1},2)
                pick2{k}(:,[(ii-1)*Enum + [1 2]]) = repmat(pick2{k-1}(:,ii),1,Enum).*[(E_STATE(:,t+k)==1) (E_STATE(:,t+k)==2)];
                end
                
                cp_tmptmp{k} = [Cpr_nb{k,T_STATERe(1,t)}(:,Agg_pick2{k-1}) Cpr_nb{k,T_STATERe(1,t)+ZtotalRe}(:,Agg_pick2{k-1})];
                cpr_tmpsim{kk,k}   = sum(interp1(mgrid2,cp_tmptmp{k},C_STATE_N5(:,kk),'linear','extrap').*pick2{k-1},2);
            end
            cp_tmptmp{K+1} = [Cpr_b{T_STATERe(1,t)}(:,Agg_pick2{k}) Cpr_b{T_STATERe(1,t)+ZtotalRe}(:,Agg_pick2{k})];
            cpr_tmpsim{kk,K+1}   = sum(interp1(mgrid2,cp_tmptmp{K+1},C_STATE_N5(:,kk),'linear','extrap').*pick2{k},2);
            C_STATE_N5(:,kk) = cpr_tmpsim{kk,K+1};
            for kkk=1:K
            cpr_tmpN5(:,kkk) = cpr_tmpsim{kkk,Choose(kkk,kk)};
            end            
            C_STATE_TMP(:,5) = mean(cpr_tmpN5,2);   
      end     
      %C_STATE_TMP = min(C_STATE_TMP,1.5*Mmax); 
      h_update(t) = mean(C_STATE_TMP(:,1))*omega1 + mean(C_STATE_TMP(:,2))*omega2 +  mean(C_STATE_TMP(:,3))*omega3+...
                    mean(C_STATE_TMP(:,4))*omega4 + mean(C_STATE_TMP(:,5))*omega5;
      C_STATE_TMP = C_STATE_TMP/h_update(t); 
      H_STATE(1,t+1) = h_update(t)*H(T_STATERe(1,t),Z_STATE(1,t+1)); 
  end
toc

%==========================================================================
% Compute the new H, Drop first TT period
%==========================================================================
     TT      = 500;
     H_STATE = H_STATE(TT+1:T-K) ;
     for i=1:ZtotalRe
         for j=1:Znum
             index      = find(((T_STATERe(TT:T-K-1)'==i).*(Z_STATE(TT+1:T-K)'==j))==1);
             datanum(i,j) = length(index);
             if isempty(index);           
                 H_new(i,j) = 1;
             else
                 H_new(i,j) = mean(H_STATE(index));
             end
         end
     end
     
%==========================================================================
% Compare the new H and old H. Update the H
%==========================================================================
      gap_H(count2)   = max(max(abs(H_new-H)))      
      if gap_H(count2)>0.01 
         H  = 1*H + 0*H_new
       else
         H  = 1*H + 0*H_new
      end
      count2  = count2+1;
end
%==========================================================================
% Allocation errors
%    sup_H_error: the sup norm of realization h
%    std_H_error: the standard deviation of realization h
%==========================================================================
  for i=1:ZtotalRe
         for j=1:Znum
             index      = find(((T_STATERe(TT:T-K-1)'==i).*(Z_STATE(TT+1:T-K)'==j))==1);
             if isempty(index);
                 sup_H_error(i,j)=0;
                 std_H_error(i,j)=0;
             else
                 sup_H_error(i,j)=max(abs(H_STATE(index)-mean(H_STATE(index))));
                 std_H_error(i,j)=std(H_STATE(index));
             end
         end
  end
  H_error(case_ind,1) = max(max(sup_H_error./H)*100);
  A_error(case_ind,1) = std(log(h_update(TT+1:T)))*100;
%==========================================================================
% Save Optimal Results (Variables and functions)
%==========================================================================
if case_ind~=1
    load functions
else
    Sav_functions = cell(6,size(MM,2));
    Cpr_functions = cell(6,size(MM,2));
end

Sav_functions{1,case_ind} = Sav_com;
Sav_functions{2,case_ind} = Sav_z;
Sav_functions{3,case_ind} = Sav_cap;
Sav_functions{4,case_ind} = Sav_bond;
Sav_functions{5,case_ind} = Sav_pr_b;
Sav_functions{6,case_ind} = Sav_pr_nb;

Cpr_functions{1,case_ind} = Cpr_com;
Cpr_functions{2,case_ind} = Cpr_z;
Cpr_functions{3,case_ind} = Cpr_cap;
Cpr_functions{4,case_ind} = Cpr_bond;
Cpr_functions{5,case_ind} = Cpr_b;
Cpr_functions{6,case_ind} = Cpr_nb;

H_all{1,case_ind} = H;
save functions Sav_functions Cpr_functions
clear Sav_functions Cpr_functions
save H_all H_all MM H_error l L port_share Choose K Income_total H_error A_error


end
%end the loop of case_ind
 